/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//                INTEL CORPORATION PROPRIETARY INFORMATION                //
//   This software is supplied under the terms of a license agreement or   //
//  nondisclosure agreement with Intel Corporation and may not be copied   //
//   or disclosed except in accordance with the terms of that agreement.   //
//       Copyright (c) 2003 Intel Corporation. All Rights Reserved.        //
//                                                                         //
//  File:      AStructureLearning.cpp                                      //
//                                                                         //
//  Purpose:   Test on structure learning of BNet                          //
//                                                                         //
//  Author(s):                                                             //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////
#include "pnl_dll.hpp"
#include <stdlib.h>
#include <stdio.h>
#include "pnlMlStaticStructLearn.hpp"
#include "pnlMlStaticStructLearnHC.hpp"

#define _OPT
#ifdef _MPI
#include "mpiwrap.h"
#endif

#ifdef _OPT
#define ALIGN_BYTES		64
//#define LEARN_GENENET_FROM_CHAIN

char *alignMalloc( int len  )
{
	char *oriP = (char *)calloc(len + ALIGN_BYTES, sizeof (char));
	char *alignP = oriP + ALIGN_BYTES - ((long)oriP & (ALIGN_BYTES-1)); 
	((void **)alignP)[-1] = oriP;
	return alignP;
}

void alignFree(void * p)
{
	free(((void **)p)[-1]);
}

#endif
PNL_USING
CBNet* CreateGeneNet(const int nnodes, int maxcpd)
{
	const int numberOfNodeTypes = nnodes ;
	
	int i;

	CGraph *pGraph = CGraph::Create(0,NULL,NULL,NULL);

	pGraph->AddNodes( nnodes );

#ifdef LEARN_GENENET_FROM_CHAIN
    for(i=0; i<nnodes-1; i++)
    {
		pGraph->AddEdge(i, i+1, 1);
    }
#endif

	CNodeType *nodeTypes = new CNodeType [numberOfNodeTypes];
	for ( i = 0 ; i < numberOfNodeTypes; i++ )
	{
		nodeTypes[i].SetType(1,3);
	}
		
	int *nodeTypeAssociation = new int [nnodes];
	for ( i = 0; i < nnodes; i++ )
	{
		nodeTypeAssociation[i] = i;
	}

	//the graph model built over, the following will build the CPD and BN

	CBNet *pBNet = CBNet::Create( nnodes, numberOfNodeTypes, nodeTypes, nodeTypeAssociation, pGraph);

#ifndef LEARN_GENENET_FROM_CHAIN
	CModelDomain *pMD = pBNet->GetModelDomain();

	CFactor **myParams = new CFactor* [nnodes];

	
	int **domain = new int *[nnodes] ;
	int *domainpoint1 = new int [1];
	int *domainpoint;
	domain[0] = domainpoint1;
	domainpoint1[0] = 0;
	for ( i = 1; i < nnodes; i++)
	{
		domainpoint = new int [1];
		domainpoint[0] = i;
		domain[i] = domainpoint;
	}

	int *nodeNumbers = new int[nnodes];
	nodeNumbers[0] = 1;
	for ( i = 1; i < nnodes; i++ )
	{
		nodeNumbers[i] =  1;
	}
#endif

	pBNet->AllocParameters();

#ifndef LEARN_GENENET_FROM_CHAIN
	for( i = 0; i < nnodes; i++ )
	{
		myParams[i] = CTabularCPD::Create( domain[i], nodeNumbers[i], pMD);
	}
#endif

	// floatVector data;
	// data.assign(maxcpd, 0.0f);

	float *data = new float[maxcpd];
	for (i = 0; i < maxcpd; i++)
		data[i] = 0.0f;
	
	for ( i = 0; i < nnodes; i++ )
	{
#ifndef LEARN_GENENET_FROM_CHAIN
		myParams[i]->AllocMatrix(data, matTable);
		pBNet->AttachParameter(myParams[i]);
#else
		pBNet->AllocParameter( i );
		(pBNet->GetFactor( i )) ->AllocMatrix( data, matTable );
#endif
	}

	delete [] nodeTypes;
#ifndef LEARN_GENENET_FROM_CHAIN
	for(i=0; i<nnodes; i++)
	{
		delete domain[i];
	}
	delete[] domain;
	delete nodeNumbers;
#endif
	return pBNet;
}

int main(int argc, char *argv[])
{

	int i, j;
	const int nnodes = 2400 ;
	//const int nnodes = 600 ;
	int ncases = 173;
	int numNt = nnodes;
	//timer_start(0);
	CBNet *bNet = CreateGeneNet(nnodes, 2000);
	

	int* obs_nodes = new int[nnodes];
	for(i=0; i<nnodes; i++)obs_nodes[i] = i;


    FILE * fp = fopen(argv[1], "r");
    //FILE * fp = fopen("c:\\data.txt", "r"); //lai, 04-27-2004, for Huwei new Data
    if( !fp )
    {
        printf("can't open cases file %s\n", argv[1]);
        exit(1);
    }

	int* raw_data = (int *)malloc(nnodes * ncases * sizeof(int));  //lai, memory
    
    int val;
    for(i=0 ; i<nnodes; i++)
    {
        for (j=0; j<ncases ; j++)
        {
            fscanf(fp, "%d,", &val);
			//raw_data[i * ncases + j] = val - 1; //lai, memory
			raw_data[i * ncases + j] = val ; //lai, memory, 04-27-2004, for Huwei New Data
        }
    }
    fclose(fp);

	CEvidence **pvEvidences = new CEvidence *[ncases];
	int dataSize = nnodes;//summ all sizes
	valueVector input_data;
	// input_data.assign(nnodes);
	input_data.resize(nnodes);

#ifdef _OPT
	Value *raw_matrix;
#ifdef _SIMD_SCORE
	raw_matrix = (Value *)alignMalloc(sizeof(Value) * (ncases + 4 - (ncases & 3)) * dataSize);
#else
	raw_matrix = (Value *)alignMalloc(sizeof(Value) * ncases * dataSize);
#endif

	CNodeValues::SetRawDataMatrix(raw_matrix, dataSize, ncases);
	
	int *obsData;
	// Eric, memory, 4/7/2004
	obsData = (int *) alignMalloc (sizeof(int) * dataSize);
#endif

    for(i=0 ; i<ncases; i++)
    {
        for (j=0; j<nnodes ; j++)
        {
            val = raw_data[j * ncases + i];  //lai, memory
            //val = raw_data[j][i];
            input_data[j].SetInt(val);
        }
        pvEvidences[i] = CEvidence::Create(bNet, nnodes, obs_nodes, input_data);
#ifdef _OPT
		pvEvidences[i]->SetEvidenceToMatrix(input_data, i); 
		pvEvidences[i]->SetObsData (obsData);  //lai, memory
#endif
    }

    //lai, memory
    free(raw_data);   

	intVector vA;
	intVector vD;

#ifdef _MPI
	pnlMpiInit(argc, argv);
#endif
	
	CMlStaticStructLearn *pLearn = CMlStaticStructLearnHC::Create(bNet, itStructLearnML, 
		 StructLearnHC, BIC, 10, vA, vD, 1);

	pLearn -> SetData(ncases, pvEvidences);
	//pLearn->SetMaxIterIPF(600);
	pLearn->SetMaxIterIPF(300);
	static_cast<CMlStaticStructLearnHC*> (pLearn) ->SetMinProgress((float)1e-5);

	//timer_stop(0);
        //float time1 = timer_read(0);

	printf("Begin Learning ...\n");
        //timer_start(1);

	pLearn ->Learn();

        //timer_stop(1);
	//float time2 = timer_read(1);


#ifdef _MPI
	pnlMpiFinalize();
	// VT_finalize();
#endif

	//////////////////////////////////////////////////////////////////////
	//const CDAG* pDAG = pLearn->GetResultDAG();
	//pDAG->Dump();
	delete pLearn;
	for( i = 0; i < ncases; i++)
	{
		delete pvEvidences[i];
	}
	delete[] pvEvidences;
	
	// Eric, memory, 4/7/2004
	// for( i = 0; i < 620000; i++)
	// {
	// 	delete pEvidencesk[i];
	// }
	// delete[] pEvidencesk;
	// delete[] samplelable;
	// End

	delete obs_nodes;

#ifdef _OPT
	alignFree ((void *)raw_matrix);
	// alignFree (offset);
	alignFree ((void *)obsData);
#endif

	//printf("Data preparation Time : %.3f\n", time1);
	//printf("Structure learning Time : %.3f\n", time2);

	return 0;
}

