/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//                INTEL CORPORATION PROPRIETARY INFORMATION                //
//   This software is supplied under the terms of a license agreement or   //
//  nondisclosure agreement with Intel Corporation and may not be copied   //
//   or disclosed except in accordance with the terms of that agreement.   //
//       Copyright (c) 2003 Intel Corporation. All Rights Reserved.        //
//                                                                         //
//  File:      pnlLearningEngine.hpp                                       //
//                                                                         //
//  Purpose:   The base class for all learning algorithms                  //
//                                                                         //
//  Author(s):                                                             //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////


#ifndef __PNLLEARNINGENGINE_HPP__
#define __PNLLEARNINGENGINE_HPP__

#include "pnlObject.hpp"
#include "pnlGraphicalModel.hpp"

PNL_BEGIN

typedef PNL_API enum
{
    itParamLearnML,
    itParamLearnEM,
    itParamLearnBayes,
    itStructLearnBIC,
    itDBNParamLearnEm,
    itStructLearnML,
    itDBNStructLearnML
} ELearningTypes;

class PNL_API CLearningEngine : public CPNLBase
{
public:
    virtual ~CLearningEngine();
    virtual void Learn()=0;
    
#ifdef PNL_OBSOLETE
    virtual void GetCriterionValue( int *nstepsOut, const float **scoreOut ) const;
#endif
    void ClearStatisticData();
    virtual void GetCriterionValue( floatVector *critValuesOut ) const;
    CGraphicalModel *m_pGrModel;  //lai, 05-17-2004, multi-threading

protected:
    CLearningEngine( CGraphicalModel *pGrModel, ELearningTypes learnType );
    bool IsInfNeed( const CEvidence * pEvidences );
    static void CheckModelValidity(const CGraphicalModel *pGrModel);
    
    //CGraphicalModel *m_pGrModel;
    ELearningTypes m_LearnType;
    floatVector m_critValue;
};



PNL_END

#endif // __PNLLEARNINGENGINE_HPP__
