/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//                INTEL CORPORATION PROPRIETARY INFORMATION                //
//   This software is supplied under the terms of a license agreement or   //
//  nondisclosure agreement with Intel Corporation and may not be copied   //
//   or disclosed except in accordance with the terms of that agreement.   //
//       Copyright (c) 2003 Intel Corporation. All Rights Reserved.        //
//                                                                         //
//  File:      pnlModelDomain.hpp                                          //
//                                                                         //
//  Purpose:   CModelDomain class definition                               //
//                                                                         //
//  Author(s):                                                             //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

#ifndef __PNLMODELDOMAIN_HPP__
#define __PNLMODELDOMAIN_HPP__

//#include "pnlFactor.hpp"
#include "pnlReferenceCounter.hpp"
//#include "pnlNodeType.hpp"
#include "pnlTypeDefs.hpp"
#include "pnlNodeType.hpp"


PNL_BEGIN

class CFactor;
class CGraphicalModel;

#ifdef SWIG
%rename (CreateAllVarsOfTheSameType) CModelDomain::Create(int, const CNodeType&, CGraphicalModel*);
%rename (GetTypesOfVars) CModelDomain::GetVariableTypes(intVector&, pConstNodeTypeVector*) const;
#endif


class PNL_API CModelDomain : public CReferenceCounter
{
public:
    //the class is derived from CReferenceCounter to store the pointers to Graphical Model
    static CModelDomain* Create( const nodeTypeVector& variableTypesIn,
            const intVector& variableAssociationIn, CGraphicalModel* pCreaterOfMD = NULL );
    static CModelDomain* Create(int numVariables, 
        const CNodeType& commonVariableType = CNodeType(1,2), 
        CGraphicalModel* pCreaterOfMD = NULL);

    //methods for work with factors
    int AttachFactor(const CFactor *pFactor);
    void ReleaseFactor(const CFactor *pFactor);
    bool IsAFactorOwner(const CFactor *pFactor);

    //methods for work with variable types
    const CNodeType* GetVariableType(int nodeNumber) const;
    
    void GetVariableTypes( intVector& varsIn,
                    pConstNodeTypeVector* varTypesOut ) const;
    inline const CNodeType* GetObsGauVarType() const;
    inline const CNodeType* GetObsTabVarType() const;
    //methods to direct access to node types 
    // returns number of node types of the model 
    inline int GetNumberOfVariableTypes() const;
#ifndef SWIG
    void GetVariableTypes( pConstNodeTypeVector* varTypesOut )const;
#endif
    void GetVariableTypes( nodeTypeVector* varTypesOut )const;
    inline int GetNumberVariables() const;
    inline int GetNumVlsForNode( int nodeNumber) const;
    void GetVariableAssociations(intVector* nodeAssociationOut) const;
#ifndef SWIG
    inline const int* GetVariableAssociations()const;
#endif
    inline int GetVariableAssociation(int variable)const;

    virtual ~CModelDomain();
protected:
    //model domain is based on CNodeTypes
    CModelDomain( const nodeTypeVector& variableTypes,
            const intVector& variableAssociation,
            CGraphicalModel* pCreaterOfMD = NULL );
    //model domain with one node type for all nodes
    CModelDomain( int numVariables, const CNodeType& commonVariableType,
            CGraphicalModel* pCreaterOfMD = NULL);
private:
    // heap of pointers to all factors created for this model domain
    pConstFactorVector m_factorsHeap;
    
    // numbers of heap's cells, where a pointer to a factor can be stored
    intQueue m_freeNumsInHeap;
    
    // evidence of some sort
    
    //variable types and association to variable types
    nodeTypeVector m_variableTypes;
    intVector m_variableAssociation;
    
    CNodeType m_obsGauVarType;
    CNodeType m_obsTabVarType;

    bool      m_bSelfCreated;
};

#ifndef SWIG

inline const CNodeType* CModelDomain::GetObsGauVarType() const
{
    return &m_obsGauVarType;
}

inline const CNodeType* CModelDomain::GetObsTabVarType() const
{
    return &m_obsTabVarType;
}

inline int CModelDomain::GetNumberOfVariableTypes() const
{
    return m_variableTypes.size();
}

inline int CModelDomain::GetNumberVariables() const
{
    return m_variableAssociation.size();
}
inline const int* CModelDomain::GetVariableAssociations()const
{
    return &m_variableAssociation.front();
}

inline int CModelDomain::GetVariableAssociation( int variable) const
{
    PNL_CHECK_RANGES( variable, 0, m_variableAssociation.size() );

    return m_variableAssociation[variable];
}

inline int CModelDomain::GetNumVlsForNode( int nodeNumber ) const
{
    const CNodeType* nt = GetVariableType(nodeNumber);
    if( nt->IsDiscrete() )
    {
        return 1;
    }
    return nt->GetNodeSize();
    
}


#endif

PNL_END

#endif // __PNLMODELDOMAIN_HPP__
