/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//                INTEL CORPORATION PROPRIETARY INFORMATION                //
//   This software is supplied under the terms of a license agreement or   //
//  nondisclosure agreement with Intel Corporation and may not be copied   //
//   or disclosed except in accordance with the terms of that agreement.   //
//       Copyright (c) 2003 Intel Corporation. All Rights Reserved.        //
//                                                                         //
//  File:      pnlNodeType.hpp                                             //
//                                                                         //
//  Purpose:   CNodeType class definition                                  //
//                                                                         //
//  Author(s):                                                             //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////
#ifndef __PNLNODETYPE_HPP__
#define __PNLNODETYPE_HPP__

#include "pnlObject.hpp"
#include "pnlException.hpp"

PNL_BEGIN

/* a class to represent node types structure and operations */
class PNL_API CNodeType : public CPNLBase
{
public:

	inline bool IsDiscrete() const;
	
	inline int GetNodeSize() const;
	
	inline void SetType( bool IsDiscrete, int nodeSize );

	inline bool operator==(const CNodeType &ntIn) const;

	inline bool operator!=(const CNodeType &ntIn) const;
	
	explicit CNodeType( bool IsDiscrete = 0, int nodeSize = -1 )
#ifndef SWIG
                        : m_IsDiscrete( IsDiscrete ), m_nodeSize(nodeSize)
	{}
#else
	;
#endif

private:
	
	bool m_IsDiscrete;
	int m_nodeSize;
};

#ifndef SWIG

inline bool CNodeType::IsDiscrete() const
{
	return m_IsDiscrete;
}

inline int CNodeType::GetNodeSize() const
{
	return m_nodeSize;
}
 
inline void CNodeType::SetType( bool IsDiscrete, int nodeSize )
{
	/* bad-args check */
	PNL_CHECK_LEFT_BORDER( nodeSize, 0 );
 	/* bad-args check end */

	m_IsDiscrete = IsDiscrete;
	m_nodeSize = nodeSize;
}

inline bool CNodeType::operator==(const CNodeType &nt) const
{
	return ( m_IsDiscrete == nt.m_IsDiscrete )
		&& ( m_nodeSize == nt.m_nodeSize );
}

inline bool CNodeType::operator!=(const CNodeType &nt) const
{
	return !operator==(nt);
}

#endif

PNL_END

#endif // __PNLNODETYPE_HPP__
