/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//                INTEL CORPORATION PROPRIETARY INFORMATION                //
//   This software is supplied under the terms of a license agreement or   //
//  nondisclosure agreement with Intel Corporation and may not be copied   //
//   or disclosed except in accordance with the terms of that agreement.   //
//       Copyright (c) 2003 Intel Corporation. All Rights Reserved.        //
//                                                                         //
//  File:      pnlNodeValues.hpp                                           //
//                                                                         //
//  Purpose:   CNodeValues class definition                                //
//                                                                         //
//  Author(s):                                                             //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////
// pgmNodeValues.hpp interface for class CNodeValues

#ifndef __PNLNODEVALUES_HPP__
#define __PNLNODEVALUES_HPP__

#include "pnlObject.hpp"
#include "pnlStaticGraphicalModel.hpp"
#include "pnlTypeDefs.hpp"
#include "pnlNodeType.hpp"

#define _OPT

#ifdef _OPT
#include "pnlArray.hpp"
#endif
PNL_BEGIN

#ifdef SWIG
%rename(GetValueBySerialNumberC) CNodeValues::GetValueBySerialNumber(int SerialNumber ) const;
#endif

class PNL_API CNodeValues : public CPNLBase
{
public:
    static CNodeValues* Create( const pConstNodeTypeVector& obsNodesTypesIn,
        const valueVector& obsValsIn );
    void ToggleNodeStateBySerialNumber( const intVector& numsOfNdsIn );
#ifdef PNL_OBSOLETE
	static CNodeValues* Create(int NObsNodes, 
						const CNodeType* const* obsNodesTypes,
                        const valueVector& obsValsIn );
    void ToggleNodeStateBySerialNumber( int nNodes, const int *nodeNumbers );
#endif //PNL_OBSOLETE

	void SetData( const valueVector& dataIn );
// you should put in nodeNumbers numbers from the model instead of indices in m_obsNodes
// const intVector *_pgmGetObsNodes()const;
	int GetNumberObsNodes() const;
	const int *GetObsNodesFlags() const;
	const int *GetOffset() const;
    void GetRawData(valueVector* valuesOut) const;
	const CNodeType *const*GetNodeTypes() const;
	inline Value const* GetValueBySerialNumber( int SerialNumber ) const;
	inline Value* GetValueBySerialNumber( int SerialNumber );
	inline int IsObserved(int SerialNumber) const;
	inline void MakeNodeHiddenBySerialNum( int serialNumber );
	inline void MakeNodeObservedBySerialNum( int serialNumber );
	virtual ~CNodeValues();

#ifdef _OPT
	void SetRawDataAddr (Value *rawData, int n) {m_rawValues.setaddr(rawData); m_rawValues.setcount(n); }
	void SetOffsetData (int *offset, int n);
	void SetObsData (int *obsData);

	int		EvSerialNum;   
	static int	generalNnodes; 
	static int	EvidenceNum;   
#ifdef _SIMD_SCORE
	static int  EvidenceNum_t4;  
#endif

	static Value	*m_rawMatrix;   
	static void	SetRawDataMatrix(Value *rawData, int nNodes, int EvidenceNum);
	void		SetEvidenceToMatrix(const valueVector& data, int EvSerialNum);
	static Value*	GetValueBySN( int SerialNumber, int evNum );
#endif

protected:
	CNodeValues ( int nNodes,const CNodeType * const*ObsNodeTypes,
							const valueVector& pValues);	
	/*When Values is creating all nodes from obsNodes are in m_isObsNow*/
	
#ifndef _OPT
	// Eric, memory, 4/7/2004
	int m_numberObsNodes; /*Number of all potentially observed nodes*/
	pConstNodeTypeVector m_NodeTypes;
	//pointers to Node Types in Model Domain
	// End
	valueVector m_rawValues;
	//vector of unions with values, type of value determine by type of node
	intVector m_offset;
	//vector of offsets in m_rawValues to value of i's node*/
	intVector m_isObsNow;
	//some nodes actually observed, some - potentially, here are the flags
#else
	// Eric, memory, 4/7/2004
	static int m_numberObsNodes;
	static pConstNodeTypeVector m_NodeTypes;
	pnlArray <Value> m_rawValues;
	static pnlArray <int>   m_offset;
	static pnlArray <int>   m_isObsNow;
	// End
#endif

private:
	
};

#ifndef SWIG

inline Value const* CNodeValues::GetValueBySerialNumber( int SerialNumber )const
{
#ifndef _OPT
//	PNL_CHECK_RANGES( SerialNumber, 0, m_numberObsNodes );
#endif

#ifdef _OPT
#ifdef _SIMD_SCORE
	return (Value* const)(m_rawMatrix + SerialNumber * EvidenceNum_t4 + EvSerialNum);
#else
	return (Value* const)(m_rawMatrix + SerialNumber * EvidenceNum + EvSerialNum);
#endif
#else
	return (Value* const)(&m_rawValues[m_offset[SerialNumber]]);
#endif
}

inline Value* CNodeValues::GetValueBySerialNumber( int SerialNumber )
{
#ifndef _OPT
// 	PNL_CHECK_RANGES( SerialNumber, 0, m_numberObsNodes );
#endif

#ifdef _OPT
#ifdef _SIMD_SCORE
	return (m_rawMatrix + SerialNumber * EvidenceNum_t4 + EvSerialNum);
#else
	return (m_rawMatrix + SerialNumber * EvidenceNum + EvSerialNum);
#endif
#else
    return (&m_rawValues[m_offset[SerialNumber]]);
#endif
}

inline int CNodeValues::IsObserved( int SerialNumber ) const
{
	return m_isObsNow[SerialNumber];
}

inline void CNodeValues::MakeNodeHiddenBySerialNum( int serialNumber )
{
	m_isObsNow[serialNumber] = 0;
}
inline void CNodeValues::MakeNodeObservedBySerialNum( int serialNumber )
{
	m_isObsNow[serialNumber] = 1;
}

#endif

PNL_END

#endif//__PNLNODEVALUES_HPP__
