#ifndef DATASET_H
#define DATASET_H

#include <bitset>
#include <algorithm>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <sstream>
#include <vector>
#include <cmath>

#include "Bitmap.h"

using namespace std;

class DataSet
{
 public:
  DataSet(string filename);

  int GetRemainingOnesInTransaction(int trans, int after) const;
  int GetTransactionCount() const;
  int GetItemCount() const;
  int GetActualIndex(int index) const;
  int * GetSupportSet(int searchFor, int *length);
  void AddItemToZeroCount(int *zeros, int item) const;
  void RemoveItemFromZeroCount(int *zeros, int item) const;
  void SetRowUnavailable(int row);
  void SetAllRowsAvailable();
  bool IsRowAvailable(int row) const;
  bool ValueAt(int row, int col) const;
  Bitmap * GetBitmaps() const;
  Bitmap * GetTransactionBitmap(int index);
  Bitmap * GetItemBitmap(int index);
  friend ostream & operator<<(ostream & out, DataSet & d);

 private:  
  vector<Bitmap> itemBits;
  vector<Bitmap> transBits;
  bool *available;


  vector< vector <int> > theData;
  unsigned int **supportBitmaps;
  int numberOfBitmaps;
  bool **data;
  int transactions;
  int items;
};

#endif

