/******************************************************************************
*                                                                             *
* This file is part of SEMPHY - Structural EM for PHYlogenetic reconstruction *
* This is unpublished proprietary source code it's authors                    *
*                                                                             *
* The contents of these coded instructions, statements and computer           *
* programs may not be disclosed to third parties, copied or duplicated in     *
* any form, in whole or in part, without the prior written permission of      *
* the authors.                                                                *
*                                                                             *
* (C) COPYRIGHT Nir Friedman, Matan Ninio,  Itsik Pe'er and Tal Pupko, 2001   *
* All Rights Reserved                                                         *
*                                                                             *
* for information please contact semphy@cs.huji.ac.il                         *
*                                                                             *
******************************************************************************/
#include "approxSemphyDistance.h"
#include "computeCounts.h"
#include "logFile.h"
#include <iostream>
#include "mpiWrap.h"
#include "wtime.h"


using namespace std;

approxSemphyDistance::approxSemphyDistance(const tree& et,
					  const sequenceContainer1G& sc,
					  const stochasticProcess& sp,
					  const computePijGam& pij0,
				      const suffStatGlobalGam& cup,
				      const suffStatGlobalGam& cdown,
				      const vector<MDOUBLE>& cprobAtEachPos,
					  const VVdouble & posteriorRateProbAtEachPos, //CODE_RED
				      const suffStatGlobalGam& computeMarginal,
				      const Vdouble* weight,
					  const MDOUBLE toll)
  : _cup(cup), _sc(sc), _sp(sp),  _cdown(cdown), _cprobAtEachPos(cprobAtEachPos),
  _posteriorRateProbAtEachPos(posteriorRateProbAtEachPos), //CODE_RED
		 _computeMarginal(computeMarginal),_et(et), _weights(weight) ,_toll(toll),_pij0(pij0) {
	

//	_ctc.countTableComponentAllocatePlace(_pi.alphabetSize(),_pi.categories());
	_distances.resize(_et.iNodes());
	_likeDistances.resize(_et.iNodes());
	for (int z=0; z< _et.iNodes(); ++z) {
		_distances[z].resize( _et.iNodes());
		_likeDistances[z].resize( _et.iNodes());
	}
}



void approxSemphyDistance::computeDistances(){
	LOG(5,<<"ENTERTING COMPUTE DISTANCES() "<<endl); // CODE_RED
	treeIterTopDownConst tIt(_et);
	tree::nodeP mynode = tIt.first();
	mynode = tIt.next(); // skipping the root
	
#ifdef _MPI
	int nnode = _et.iNodes();	
	vector<countTableComponentGam> vec;
	vec.resize(nnode);
	computeCounts cc;
	for (; mynode != tIt.end(); mynode = tIt.next()) 
		cc.fillCountTableComponentGam(vec[mynode->id()],_sp,_sc,_pij0,_cup,_cdown,_weights,mynode,_cprobAtEachPos);
	int cate, alpha1, alpha2;
	int csize = _sp.categories();
	int asize = _pij0.alphabetSize();
	int factor2 = asize * asize;
	int factor1 = csize * factor2;
	int total = nnode * factor1;
	int num = 0;
	MDOUBLE *tmp = new MDOUBLE[total];
	MDOUBLE *sum = new MDOUBLE[total];
	//for (int i = 0; i < total; ++i) sum[i] = 0;
	mynode = tIt.first(); 
	for (mynode = tIt.next(); mynode != tIt.end(); mynode = tIt.next()) {
		for (cate = 0; cate < csize; ++cate)
			for (alpha1 = 0; alpha1 < asize; ++alpha1)
				for (alpha2 = 0; alpha2 < asize; ++alpha2)
					tmp[num*factor1+cate*factor2+alpha1*asize+alpha2] = vec[mynode->id()].getCounts(alpha1, alpha2, cate);
		++num;
	}
	MPI_Allreduce(tmp, sum, total, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);
	num = 0;
	mynode = tIt.first();
	for (mynode = tIt.next(); mynode != tIt.end(); mynode = tIt.next()) {
		for (cate = 0; cate < csize; ++cate)
			for (alpha1 = 0; alpha1 < asize; ++alpha1)
				for (alpha2 = 0; alpha2 < asize; ++alpha2)
					vec[mynode->id()].setCount(alpha1, alpha2, cate, sum[num*factor1+cate*factor2+alpha1*asize+alpha2]);
		++num;
	}
	delete[] tmp;
	delete[] sum;
	--nnode;
	const MDOUBLE tollForPairwiseDist = 0.001;
	mynode = tIt.first();
	for (mynode = tIt.next(); mynode != tIt.end(); mynode = tIt.next()) {
		fromCountTableComponentToDistance from1(vec[mynode->id()],_sp,tollForPairwiseDist,mynode->dis2father());
		from1.computeDistance();
		mynode->setDisToFather(from1.getDistance());
		const tree::nodeP nodeFather = mynode->father();
		if (mynode->id()<nodeFather->id()) {
		  _distances[mynode->id()][nodeFather->id()] =
		    from1.getDistance();
		  _likeDistances[mynode->id()][nodeFather->id()] =
		    from1.getLikeDistance();
		}
		else {
		  _distances[nodeFather->id()][mynode->id()] = 
		    from1.getDistance();
		  _likeDistances[nodeFather->id()][mynode->id()] = 
		    from1.getLikeDistance();
		}
	}

/*	int nmean, nleft, startp;
    nmean = (int)(nnode / _num_procs);
    nleft = nnode % _num_procs;
    if (_my_rank < nleft)
        startp = _my_rank * (++nmean);
    else
        startp = _my_rank * nmean + nleft;
	num = 0;
	tmp = new MDOUBLE[2*nnode];
	sum = new MDOUBLE[2*nnode];
	mynode = tIt.first();
	for (mynode = tIt.next(); mynode != tIt.end(); mynode = tIt.next()) {
		if (num < startp) {tmp[num] = tmp[num+nnode] = 0; ++num; continue;}
		if (num >= startp + nmean) {tmp[num] = tmp[num+nnode] = 0; ++num; continue;}
		fromCountTableComponentToDistance from1(vec[mynode->id()],_sp,tollForPairwiseDist,mynode->dis2father());
		from1.computeDistance();
		tmp[num] = from1.getDistance();
		tmp[num+nnode] = from1.getLikeDistance();
		++num;
	}
	MPI_Allreduce(tmp, sum, 2*nnode, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);
	num = 0; mynode = tIt.first();
	for (mynode = tIt.next(); mynode != tIt.end(); mynode = tIt.next()) {
		int fatherid = mynode->father()->id();
		int sonid = mynode->id();
		mynode->setDisToFather(sum[num]);
		if (sonid<fatherid) {
			_distances[sonid][fatherid] = sum[num];
			_likeDistances[sonid][fatherid] = sum[num+nnode];
		}
		else {
			_distances[fatherid][sonid] = sum[num];
			_likeDistances[fatherid][sonid] = sum[num+nnode];
		}
		++num;
	}
	delete[] tmp;
	delete[] sum;*/
#else
#ifdef _OPENMP	
#pragma intel omp parallel taskq shared(mynode)
#endif
	{
		for (; mynode != tIt.end(); mynode = tIt.next()) {
#ifdef _OPENMP			
#pragma intel omp task captureprivate(mynode)
#endif
		  {	
			computeApproxDistancesNodesSonFather(mynode);
		  }	
		}
	}
#endif

#ifdef _MPI
	mpicomputeApproxDistancesNodes();
#else
	vector<tree::nodeP> allNodes;
	_et.getAllNodes(allNodes,_et.iRoot());

	int i,j;

#ifdef _OPENMP
	int dx = allNodes.size();
#pragma omp parallel for private(j) schedule(dynamic)
	for (i = 0; i < dx - 1; ++i) {
		for (j = i + 1; j < dx; ++j) {
#else
	for (i=0; i < allNodes.size()-1;++i) {
		for (j=i+1; j < allNodes.size(); ++j) {
#endif			
			computeApproxDistancesNodes(allNodes[i],allNodes[j]);
		}
	}
#endif
}

void approxSemphyDistance::computeApproxDistancesNodes(
						const tree::nodeP node1,
						const tree::nodeP node2) {

	if (node1->father() == node2) {
		return;
	} else if (node2->father() == node1) {
		return;
	}
	else computeApproxDistancesSeparateNodes(node1,node2);
}

void approxSemphyDistance::computeApproxDistancesNodesSonFather(
						const tree::nodeP nodeSon) {

	computeCounts cc;
	countTableComponentGam ctcGam;
	cc.fillCountTableComponentGam(ctcGam,
								_sp,//const stochasticProcess& sp,
								_sc,//const sequenceContainer1G& sc,
								_pij0,//computePijGam& pij0,
								_cup,//const suffStatGlobalGam& cup,
								_cdown,//const suffStatGlobalGam& cdown,
								_weights,//const Vdouble * weights,
								nodeSon,//tree::nodeP nodeSon,
								_cprobAtEachPos);//const Vdouble& posProbVec
	const tree::nodeP nodeFather = nodeSon->father();
	const MDOUBLE tollForPairwiseDist = 0.001;
	fromCountTableComponentToDistance from1(ctcGam,_sp,tollForPairwiseDist,nodeSon->dis2father());
	from1.computeDistance();
	nodeSon->setDisToFather(from1.getDistance()); // this is actually EM step for BBL.

	if (nodeSon->id()<nodeFather->id()) {
	  _distances[nodeSon->id()][nodeFather->id()] =
	    from1.getDistance();
	  _likeDistances[nodeSon->id()][nodeFather->id()] =
	    from1.getLikeDistance();
	}
	else {
	  _distances[nodeFather->id()][nodeSon->id()] = 
	    from1.getDistance();
	  _likeDistances[nodeFather->id()][nodeSon->id()] = 
	    from1.getLikeDistance();
	}
	//	LOG(5,<<getLikeDistance(nodeSon->id(),nodeFather->id()));
	//	LOG(5,<<endl);
	
}

void approxSemphyDistance::computeApproxDistancesSeparateNodes(
						const tree::nodeP node1,
						const tree::nodeP node2) {

	// cerr <<" ENTERING approxSemphyDistance::computeApproxDistancesSeparateNodes(... "<<endl; //CODE_RED
	countTableComponentGam ctcGam;
	ctcGam.countTableComponentAllocatePlace(_sp.alphabetSize(),_sp.categories());

#ifdef _OPT
	int alphabetSize = _sp.alphabetSize();
	int categorySize = _sp.categories();
#endif

	if (_weights!=NULL) {
#ifdef _MPI
		for (int pos=0; pos< mpi_mean; ++pos) {
			if ((*_weights)[pos+mpi_spos] == 0.0) continue; 
#else		
		for (int pos=0; pos< _sc.seqLen(); ++pos) {
			if ((*_weights)[pos] == 0.0) continue; 
#endif
#ifndef _OPT
			for (int alph1 =0; alph1< _sp.alphabetSize(); ++alph1) {
				for (int alph2 =0; alph2< _sp.alphabetSize(); ++alph2) {
					for (int rateCategor =0; rateCategor< _sp.categories(); ++rateCategor) {
#else
			for (int alph1 =0; alph1< alphabetSize; ++alph1) {
				for (int alph2 =0; alph2< alphabetSize; ++alph2) {					
					for (int rateCategor =0; rateCategor< categorySize; ++rateCategor) {
#endif
						MDOUBLE tmp = 
						  _computeMarginal.get(
										 node1->id(),
										 pos,
										 rateCategor,
										 alph1
										 ) *
						  _computeMarginal.get(
										 node2->id(),
										 pos,
										 rateCategor,
										 alph2
										 )
										  * _posteriorRateProbAtEachPos[pos][rateCategor]; // CODE_RED2!
											
#ifdef _MPI
						  ctcGam.addToCounts(alph1,alph2,rateCategor,tmp*(*_weights)[pos+mpi_spos]);
#else
						  ctcGam.addToCounts(alph1,alph2,rateCategor,tmp*(*_weights)[pos]);
#endif
					}
				}
			}
		}
	}else{		// _weights==NULL
		MDOUBLE d1, d2, d3;
#ifdef _MPI
		for (int pos=0; pos< mpi_mean; ++pos) {
#else
		for (int pos=0; pos< _sc.seqLen(); ++pos) {
#endif
#ifndef _OPT
			for (int alph1 =0; alph1< _sp.alphabetSize(); ++alph1) {
				for (int alph2 =0; alph2< _sp.alphabetSize(); ++alph2) {
					for (int rateCategor =0; rateCategor< _sp.categories(); ++rateCategor) {
#else
			for (int alph1 =0; alph1< alphabetSize; ++alph1) {
				for (int alph2 =0; alph2< alphabetSize; ++alph2) {					
					for (int rateCategor =0; rateCategor< categorySize; ++rateCategor) {
#endif
						d1 = _computeMarginal.get(pos,rateCategor,node1->id(),alph1);
						d2 = _computeMarginal.get(pos,rateCategor,node2->id(),alph2);
						d3 = _posteriorRateProbAtEachPos[pos][rateCategor];
						MDOUBLE tmp = d1 * d2 * d3;
								 
						ctcGam.addToCounts(alph1,alph2,rateCategor,tmp);
					}
				}
			}
		}
	}
#ifdef _MPI
#ifndef _OPT
int alphabetSize = _sp.alphabetSize();
int categorySize = _sp.categories();
#endif
int factor = alphabetSize * alphabetSize;
MDOUBLE *tmp = new MDOUBLE[factor*categorySize];
MDOUBLE *sum = new MDOUBLE[factor*categorySize];
for (int rateCategor =0; rateCategor< categorySize; ++rateCategor) 
	for (int alph1 =0; alph1< alphabetSize; ++alph1) 
		for (int alph2 =0; alph2< alphabetSize; ++alph2) 					
			tmp[rateCategor*factor+alph1*alphabetSize+alph2] = ctcGam.getCounts(alph1, alph2, rateCategor);	
	MPI_Allreduce(tmp, sum, factor * categorySize, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);
for (int rateCategor =0; rateCategor< categorySize; ++rateCategor) 
	for (int alph1 =0; alph1< alphabetSize; ++alph1) 
		for (int alph2 =0; alph2< alphabetSize; ++alph2) 					
			ctcGam.setCount(alph1, alph2, rateCategor, sum[rateCategor*factor+alph1*alphabetSize+alph2]);	
delete[] tmp;
delete[] sum;
#endif
	MDOUBLE initialGuess = getDistance(node1->id(),node2->id());
	if (initialGuess==0) initialGuess=0.031;
	const MDOUBLE tollForPairwiseDist = 0.001;
	fromCountTableComponentToDistance from1(ctcGam,_sp,tollForPairwiseDist,initialGuess);
	from1.computeDistance();
	if (node1->id()<node2->id()) {
		_distances[node1->id()][node2->id()] = 
			from1.getDistance();
		_likeDistances[node1->id()][node2->id()] = 
			from1.getLikeDistance();
	}
	else {
		_distances[node2->id()][node1->id()] = 
			from1.getDistance();
		_likeDistances[node2->id()][node1->id()] = 
			from1.getLikeDistance();
	}
}

#ifdef _MPI
void approxSemphyDistance::mpicomputeApproxDistancesNodes()
{
	vector<tree::nodeP> allNodes;
	_et.getAllNodes(allNodes,_et.iRoot());
	int i, j, cate, alph1, alph2, pos, num;
	int nsize = allNodes.size();
	int asize = _sp.alphabetSize();
	int csize = _sp.categories();
	int lsize = _sc.seqLen();
	int factor3 = asize * csize;
	int factor2 = factor3 * asize;
	int total = (nsize-1) * nsize * factor2 / 2;
	int total1 = nsize * factor3 * lsize;
	if (total > total1) {
		mpialternative();
		return;
	}				
	
	MDOUBLE *tmp = new MDOUBLE[total];
	MDOUBLE *sum = new MDOUBLE[total];
	for (i = 0; i < total; ++i)
		tmp[i] = sum[i] = 0;
	num = 0;
	if (_weights!=NULL) {
		for (i = 0; i < nsize - 1; ++i) {
			for (j = i + 1; j < nsize; ++j) {
				if (allNodes[i]->father() == allNodes[j] || allNodes[j]->father() == allNodes[i])
					continue;
				for (pos = 0; pos < mpi_mean; ++pos) {
					if ((*_weights)[pos+mpi_spos] == 0.0) continue;
					for (alph1 = 0; alph1 < asize; ++alph1) {
						for (alph2 = 0; alph2 < asize; ++alph2) {
							for (cate = 0; cate < csize; ++cate) {
								MDOUBLE temp = 
								  _computeMarginal.get(pos,cate,allNodes[i]->id(),alph1)
												 *
								  _computeMarginal.get(pos,cate,allNodes[j]->id(),alph2)
						 						 * 
								 _posteriorRateProbAtEachPos[pos][cate];// CODE_RED2!
								tmp[num*factor2+alph1*factor3+alph2*csize+cate] += 
									temp * (*_weights)[pos + mpi_spos];
							}
						}
					}
				}
				++num;
			}
		}
	}
	else {
		for (i = 0; i < nsize - 1; ++i) {
			for (j = i + 1; j < nsize; ++j) {
				if (allNodes[i]->father() == allNodes[j] || allNodes[j]->father() == allNodes[i])
					continue;
				for (pos = 0; pos < mpi_mean; ++pos) {
					for (alph1 = 0; alph1 < asize; ++alph1) {
						for (alph2 = 0; alph2 < asize; ++alph2) {
							for (cate = 0; cate < csize; ++cate) {
								MDOUBLE temp = 
								  _computeMarginal.get(pos,cate,allNodes[i]->id(),alph1)
												 *
								  _computeMarginal.get(pos,cate,allNodes[j]->id(),alph2)
						 						 * 
								 _posteriorRateProbAtEachPos[pos][cate];// CODE_RED2!
								tmp[num*factor2+alph1*factor3+alph2*csize+cate] += temp;
							}
						}
					}
				}
				++num;
			}
		}
	}
	MPI_Allreduce(tmp, sum, total, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);
	delete[] tmp;
	countTableComponentGam ctcGam;
	ctcGam.countTableComponentAllocatePlace(asize, csize);
	
	int nmean, nleft, startp;
	nmean = (int)(num / _num_procs);
	nleft = num % _num_procs;
	if (_my_rank < nleft)
		startp = _my_rank * (++nmean);
	else
		startp = _my_rank * nmean + nleft;
	tmp = new MDOUBLE[2*num];
	MDOUBLE *sum1 = new MDOUBLE[2*num];
//	for (i = 0; i < 2 * num; ++i) sum1[i] = 0;
	int linksize = num;
	const MDOUBLE tollForPairwiseDist = 0.001;
	
	num = 0;
	for (i = 0; i < nsize - 1; ++i) {
		for (j = i + 1; j < nsize; ++j) {
			if (allNodes[i]->father() == allNodes[j] || allNodes[j]->father() == allNodes[i])
				continue;
			if (num < startp || num >= startp + nmean) {tmp[num] = tmp[num+linksize] = 0; ++num; continue;}
			for (alph1 = 0; alph1 < asize; ++alph1) {
				for (alph2 = 0; alph2 < asize; ++alph2) {
					for (cate = 0; cate < csize; ++cate) {
						ctcGam.setCount(alph1, alph2, cate, 
								sum[num*factor2+alph1*factor3+alph2*csize +cate]);
					}
				}
			}
			MDOUBLE initialGuess = getDistance(allNodes[i]->id(), allNodes[j]->id());
			if (initialGuess == 0) initialGuess = 0.031;
			fromCountTableComponentToDistance from1(ctcGam,_sp,tollForPairwiseDist,initialGuess);
			from1.computeDistance();
			tmp[num] = from1.getDistance();
			tmp[num+linksize] = from1.getLikeDistance();
			++num;
		}
	}
	MPI_Allreduce(tmp, sum1, 2 * linksize, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);
	num = 0;
	for (i = 0; i < nsize - 1; ++i) {
		for (j = i + 1; j < nsize; ++j) {
			if (allNodes[i]->father() == allNodes[j] || allNodes[j]->father() == allNodes[i])
				continue;
			int id1 = allNodes[i]->id();
			int id2 = allNodes[j]->id();
			if (id1 < id2) {
				_distances[id1][id2] = sum1[num];
				_likeDistances[id1][id2] = sum1[num+linksize];
			}
			else {
				_distances[id2][id1] = sum1[num];
				_likeDistances[id2][id1] = sum1[num+linksize];
			}
			++num;
		}
	}
	delete[] sum1;
	delete[] sum;
}

void approxSemphyDistance::mpialternative()
{
	vector<tree::nodeP> allNodes;
    _et.getAllNodes(allNodes,_et.iRoot());
    int i, j, cate, alph1, alph2, pos, num;
    int nsize = allNodes.size();
    int asize = _sp.alphabetSize();
    int csize = _sp.categories();
    int lsize = _sc.seqLen();
	int factor2 = asize * nsize;
	int factor1 = factor2 * csize;
	int total = factor1 * lsize;
	MDOUBLE *sum1 = new MDOUBLE[lsize * csize];						
	MDOUBLE *tmp = new MDOUBLE[total];
	MDOUBLE *sum = new MDOUBLE[total];
	for (i = 0; i < total; ++i)
		tmp[i] = 0;
	for (pos = 0; pos < mpi_mean; ++pos) 
		for (cate = 0; cate < csize; ++cate) 
			tmp[(pos+mpi_spos)*csize+cate] = _posteriorRateProbAtEachPos[pos][cate];
	MPI_Allreduce(tmp, sum1, lsize*csize, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);
	for (i = 0; i < lsize*csize; ++i)
		tmp[i] = 0;
	for (pos = 0; pos < mpi_mean; ++pos) {
		for (cate = 0; cate < csize; ++cate) {
			for (i = 0; i < nsize; ++i) {
				for (alph1 = 0; alph1 < asize; ++alph1) {
					tmp[(pos+mpi_spos)*factor1+cate*factor2+i*asize+alph1] =
							_computeMarginal.get(pos, cate, allNodes[i]->id(), alph1);
				}
			}
		}
	}
	MPI_Allreduce(tmp, sum, total, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);
	delete[] tmp;
	num = 0;
	for (i = 0; i < nsize - 1; ++i)
		for (j = i + 1; j < nsize; ++j)
			if (allNodes[i]->father() != allNodes[j] && allNodes[j]->father() != allNodes[i]) ++num;
	int nmean, nleft, startp;
	nmean = (int)(num / _num_procs);
	nleft = num % _num_procs;
	if (_my_rank < nleft)
		startp = _my_rank * (++nmean);
	else
		startp = _my_rank * nmean + nleft;
	tmp = new MDOUBLE[2*num];
	MDOUBLE *sum2 = new MDOUBLE[2*num];
	int linksize = num;
	num = 0;
	const MDOUBLE tollForPairwiseDist = 0.001;
	for (i = 0; i < nsize - 1; ++i)
		for (j = i + 1; j < nsize; ++j) {
			if (allNodes[i]->father() == allNodes[j] || allNodes[j]->father() == allNodes[i]) continue;
			if (num < startp || num >= startp + nmean) {tmp[num] = tmp[num+linksize] = 0; ++num; continue;}
			countTableComponentGam ctcGam;
			ctcGam.countTableComponentAllocatePlace(asize, csize);
			if (_weights!=NULL) {
				for (pos=0; pos< lsize; ++pos) {
					if ((*_weights)[pos] == 0.0) continue; 
					for (alph1 = 0; alph1 < asize; ++alph1) 
						for (alph2 = 0; alph2 < asize; ++alph2) 
							for (cate = 0; cate < csize; ++cate) {
								MDOUBLE temp = sum[pos*factor1+cate*factor2+i*asize+alph1]
											* sum[pos*factor1+cate*factor2+j*asize+alph2]
											* sum1[pos*csize+cate];
								ctcGam.addToCounts(alph1, alph2, cate, temp*(*_weights)[pos]); ///something wrong here
							}	
				}
			}
			else {
				for (pos = 0; pos < lsize; ++pos) 
					for (cate = 0; cate < csize; ++cate) 
						for (alph1 = 0; alph1 < asize; ++alph1) 
							for (alph2 = 0; alph2 < asize; ++alph2) { 
								MDOUBLE temp = sum[pos*factor1+cate*factor2+i*asize+alph1]
											* sum[pos*factor1+cate*factor2+j*asize+alph2]
											* sum1[pos*csize+cate];
								ctcGam.addToCounts(alph1, alph2, cate, temp);
							}
			}
			MDOUBLE initialGuess = getDistance(allNodes[i]->id(), allNodes[j]->id());
			if (initialGuess == 0) initialGuess = 0.031;
			fromCountTableComponentToDistance from1(ctcGam,_sp,tollForPairwiseDist,initialGuess);
			from1.computeDistance();
			tmp[num] = from1.getDistance();
			tmp[num+linksize] = from1.getLikeDistance();
			++num;
		}
	MPI_Allreduce(tmp, sum2, 2 * linksize, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);
	delete[] sum1;
	delete[] sum;
	delete[] tmp;
	num = 0;
	for (i = 0; i < nsize - 1; ++i) {
		for (j = i + 1; j < nsize; ++j) {
			if (allNodes[i]->father() == allNodes[j] || allNodes[j]->father() == allNodes[i])
				continue;
			int id1 = allNodes[i]->id();
			int id2 = allNodes[j]->id();
			if (id1 < id2) {
				_distances[id1][id2] = sum2[num];
				_likeDistances[id1][id2] = sum2[num+linksize];
			}
			else {
				_distances[id2][id1] = sum2[num];
				_likeDistances[id2][id1] = sum2[num+linksize];
			}
			++num;
		}
	}
	delete[] sum2;

}
			
#endif




