/*
#include "uniDistribution.h"
#include "aaJC.h"
#include "nucJC.h"
#include "pijAccelerator.h"
#include "trivialAccelerator.h"
#include "chebyshevAccelerator.h"
#include "stochasticProcess.h"
#include "nucleotide.h"
#include "amino.h"
#include "sequenceContainer1G.h"
#include "phylipFormat.h"
#include "maseFormat.h"
#include "fastaFormat.h"
#include "treeInterface.h"
#include "likelihoodComputation.h"
#include "computePijComponent.h"
#include "seqContainerTreeMap.h"
#include "gammaDistribution.h"
#include "sequenceContainerNG.h"
#include "brLenOptEM.h"
#include "pupJTT.h"
#include "fromInstructionFile.h"
#include "NNiProp.h"
#include "NNiSep.h"
#include "jcDistance.h"
#include "distanceTable.h"
#include "nj.h"
#include "brLenOptEM.h"

#include "computeUpAlg.h"
#include "computeDownAlg.h"
#include "computeMarginalAlg.h"
#include "semphyStep.h"

#include <string>
#include <fstream>
#include <iostream>
#include <ctime>

using namespace std;
using namespace treeInterface;
using namespace likelihoodComputation;
using namespace brLenOptEM;
void getSEMPHY_InfoFromInsFile(const string& insFileName,
						   vector<alphabet* >& alphabets,
							vector<sequenceContainer1G>& scVec,
							vector<tree>& myet,
							vector<char>& idNotToSwap,
							vector<stochasticProcess>& spVec,
							string& outputFileName) {
	fromInstructionFile f(insFileName);
	f.setLogFile();

	outputFileName = f.getOutFile();


	f.getAlphabets(alphabets);
	f.getStartingSequenceData(scVec,alphabets);
	bool findTree = f.getStartingEvolTrees(myet,idNotToSwap);
	if (!findTree) errorMsg::reportError("problems finding the tree in function mulAnalysis::mulAnalysis");

	if (myet.size() != 1) errorMsg::reportError("problems finding the tree in function mulAnalysis::mulAnalysis - more than one tree in treefile");
	for (int z=1; z < scVec.size(); ++z) myet.push_back(myet[0]);
	f.getStartingStochasticProcess(spVec);
	if (f.useGamma()) f.getStartingGammaParameters(spVec);
}

//void semphyFromInsFile(int argc, char* argv[]){
void mainX(int argc, char* argv[]){
	cerr<<"THIS PROGRAM COMPUTE THE SEMPHY TREE "<<endl;
	cerr<<"(IN CASE OF MANY GENES, IT CONCATANATES. HERE, THERE IS NO OPTIMIZATION"<<endl;
	cerr<<"OF ALPHA PARAMETER. IF THESE VALUES"<<endl;
	cerr<<"ARE NOT GIVEN IN THE COMMAND FILE, IT WILL BE A HOMOGENOUS MODEL (NO ALPHA)"<<endl;
	cerr<<"LAST MODIFIED: 11 May 2002"<<endl;


	if (argc != 2) errorMsg::reportError("The program must get one parameter\
	\n in the command line - the instruction file name");

	
	string insFileName = argv[1];
	vector<alphabet* > alphabets;
	vector<sequenceContainer1G> scVec;
	vector<tree> myet;
	vector<char> idNotToSwap;
	vector<stochasticProcess> spVec;
	string outputFileName;
	getSEMPHY_InfoFromInsFile(insFileName,alphabets,scVec,myet,idNotToSwap,spVec,outputFileName);
	ofstream out(outputFileName.c_str());


	tree et = myet[0];
	sequenceContainer1G sc = scVec[0];
	for (int k=1; k < scVec.size(); ++k) sc.addFromSequenceContainer1G(scVec[k]);
	stochasticProcess sp = spVec[0];

	if ((spVec.size() != 1) || (myet.size() != 1) || (alphabets.size() != 1)) {
		errorMsg::reportError("semphy is not implemented for other than the concatanate model");
	}

	optimizeBranchLength1G_EM(et,sc,sp,NULL);

	for (int z=0; z < 1000; ++z) {
		tree bestTree = et;
		out<<"semphy iteration: "<<z<<endl;
		cerr<<"semphy iteration: "<<z<<endl;
		computePijGam pij0;
		pij0.fillPij(et,sp,alphabets[0]->size());

		suffStatGlobalGam cup;
		suffStatGlobalGam cdown;
		suffStatGlobalGam cmarg;

		cup.allocatePlace(sc.seqLen(),sp.categories(),et.iNodes(),sc.alphabetSize());
		cdown.allocatePlace(sc.seqLen(),sp.categories(),et.iNodes(),sc.alphabetSize());
		cmarg.allocatePlace(sc.seqLen(),sp.categories(),et.iNodes(),sc.alphabetSize());

		computeUpAlg cupAlg;
		computeDownAlg cdownAlg;
		computeMarginalAlg cmargAlg;
		int pos = 0;
		int categor = 0;

		for (pos = 0; pos < sc.seqLen(); ++pos) {
			for (categor = 0; categor < sp.categories(); ++categor) {
				cupAlg.fillComputeUp(et,sc,pos,pij0[categor],cup[pos][categor]);
				cdownAlg.fillComputeDown(et,sc,pos,pij0[categor],cdown[pos][categor],cup[pos][categor]);
			}
		}

		out<<"starting semhy step by bbl...";
		cerr<<"starting semhy step by bbl...";
		

		Vdouble posLike;

		MDOUBLE tLike = likelihoodComputation::getTreeLikelihoodFromUp2(et,
							sc,
							sp,
							cup,
							posLike, // fill this vector with each position likelihood but without the weights.
							NULL);//const Vdouble * weights=0);

		cerr<<"starting likelihood = "<<tLike<<endl;
		out<<"starting likelihood = "<<tLike<<endl;
		MDOUBLE oldL = tLike;

		for (pos = 0; pos < sc.seqLen(); ++pos) {
			MDOUBLE posPr = exp(posLike[pos]);
			for (categor = 0; categor < sp.categories(); ++categor) {
				cmargAlg.fillComputeMarginal(et,sc,sp,pos,pij0[categor],cmarg[pos][categor],cup[pos][categor],cdown[pos][categor],posPr);
			}
		}



		semphyStep semphyStep1(
			  et,//tree& et,
			  sc,//const sequenceContainer1G& sc,
			  sp,//const stochasticProcess& sp,
			  pij0,//const computePijGam& pij0,
			  cup,//const suffStatGlobalGam& cup,
			  cdown,//const suffStatGlobalGam& cdown,
			  true,//const bool useApproxCounts,
			  posLike,//const vector<MDOUBLE>& cprobAtEachPos,
			  cmarg,//const suffStatGlobalGam& computeMarginal,
				NULL,//		const Vdouble *weights,
				0.001);//	const MDOUBLE toll);



	//xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx finding the likelihood of the final trees, and output		


		MDOUBLE resx =
				optimizeBranchLength1G_EM(et,sc,sp,NULL);

		cerr<<"likelihood after semphy step is: "<<resx<<endl;
		out<<"likelihood after semphy step is: "<<resx<<endl;
		
		out<<"sep best trees under semphy (gam)"<<endl;
		et.output(out);
		
		bool sameT = treeInterface::sameTreeTolopogy(bestTree,et);

		if (sameT) break;
		if (resx < oldL + 0.05) break;
	}

	out<<" ============== final tree and likelihoods ================== "<<endl;
	cerr<<" ============== final tree and likelihoods ================== "<<endl;
		MDOUBLE resF =
				optimizeBranchLength1G_EM(et,sc,sp,NULL);

		cerr<<"likelihood after semphy step is: "<<resF<<endl;
		out<<"likelihood after semphy step is: "<<resF<<endl;
		
		out<<"Best trees under semphy (gam)"<<endl;
		et.output(out);
}
*/
