#include <algorithm>
#include "bootstrap.h"

bootstrap::bootstrap(const treeVec& treevect):_numTrees(0), _lastTaxaId(0), _nTaxa(0), _reverseIsUpToDate(false){
  fillFromTreeVec(treevect);
}

bootstrap::bootstrap (const string& filename):_numTrees(0), _lastTaxaId(0), _nTaxa(0), _reverseIsUpToDate(false){
  fillFromTreeVec(getStartingTreeVecFromFile(filename));
}


vector<float> bootstrap::GetWeightsToTree(const tree& inTree) const {
  vector<float> v(inTree.iNodes());
  recursivlyFillBTVector(inTree.iRoot(), v);
  return(v);
}

struct ltsplit
{
  bool operator()(const split& s1, const split& s2) const
  {
    return s1.size() < s2.size() ;
  }
};

vector<float> bootstrap::thresholdTree(const tree& outTree, set<pair<int, int> > &edges, const float threshold/*=0.5*/) const
{
  // reverse
  vector<split> consensus;
  vector<float> tmp(_nTaxa*2,0.0);
  edges.clear();
  pair<int,int> newpair(-1,-1);
  
  getTreeNodes(outTree);
  splitMap::reverse_sMap_t::const_reverse_iterator rmapConstRend = _rmap.rend();
  for (splitMap::reverse_sMap_t::const_reverse_iterator i=_rmap.rbegin();
       i != rmapConstRend;
       ++i) 
    {
      bool compatible(true);
      float support = 1.*i->first/_numTrees;
      
      if (support >= threshold) {
	for (vector<split>::const_iterator j=consensus.begin(); j != consensus.end(); ++j) 
	  if (!(i->second.compatible(*j))){
	    compatible=false;
	    break;
	  }
	if (compatible) {
	  consensus.push_back(i->second);
	}
      }
    }
  // i need to impliment a "consensus tree reconstruction", but untill I do, this will have to do.
  // initilise the "head of clan" vector, all in local IDs
  vector<int> head(_nTaxa,0);
  for (int j=0 ;j<_nTaxa;++j)
    head[j]=j;

  // trush
  //  print_names(cout);

  int nextId=_nTaxa;
  
  // sort consensus by split size
  sort(
       consensus.begin(),
       consensus.end(),
       ltsplit()
       );


  // the fun begins:
  for (vector<split>::iterator i1 = consensus.begin();i1!=consensus.end();++i1) {
    // take a split
    //    cout <<*i1<< endl;
    vector<int> id;
    i1->getId(id);
    // for each of it's members (in the smaller side of the split)
    for (vector<int>::iterator i2 = id.begin();i2!=id.end();++i2)
      {
	//	cout << *i2 << ":";
	newpair.first=getIdFromTree(nextId);
	//      if (head[*i2]!=*i2)
	newpair.second=getIdFromTree(head[*i2]);
	edges.insert(newpair);
	//	cout << "<<"<<newpair.first<<","<<newpair.second<<">>"<<endl;
	head[*i2]=nextId;
      }
    //    cout <<endl;
    tmp[getIdFromTree(nextId)]=((float)(_Splits.check(*i1)))/_numTrees;	// ??????
    ++nextId;
    //    for (int jj=0 ;jj<head.size();++jj) {
    //      cout << "<*<"<<jj<<","<<head[jj]<<">*>  ";
    //    }
    //    cout <<endl;
    
  }
  for (vector<int>::iterator j_final=head.begin() ;j_final!=head.end();++j_final) {
    newpair.first=getIdFromTree(nextId);
    newpair.second=getIdFromTree(*j_final);
    edges.insert(newpair);
    //    cout << "<<"<<newpair.first<<","<<newpair.second<<">>"<<endl;
  }

  return (tmp);
}

void bootstrap::print(ostream& sout/* = cout*/)
{
  _Splits.print(sout);
}

void bootstrap::printTreeNH(ostream &os, const tree &t, const vector<float> &v) const{
  os <<"(";
  vector<tree::nodeP>::iterator i=t.iRoot()->sons.begin();
  recursivlyPrintTreeNH(os, *i,v, t.iLeaves());
  for (++i;i!=t.iRoot()->sons.end();++i) {
    os <<",";
    recursivlyPrintTreeNH(os, *i,v, t.iLeaves());
  }
  os <<")";
  if (v[t.iRoot()->id()]>0)
  os << "["<<v[t.iRoot()->id()]<<"]";
  os<<";";

}

void bootstrap::fillFromTreeVec(const treeVec& treevect) 
{
  for (treeVec::const_iterator i=treevect.begin();i!=treevect.end();++i)
    splitTree(*i);
  computeReverse();
}

void bootstrap::computeReverse(){
  if (!_reverseIsUpToDate) {
    _rmap =_Splits.reverse();
    _reverseIsUpToDate = true;
  }
}

void bootstrap::print_names(ostream &os) const {
  NameMap_t::const_iterator i(_nameMap.begin());
  for (;i!=_nameMap.end();++i)
    cout << "{"<<i->first<<" = "<<i->second<<"}"<<endl;

}

int bootstrap::idFromName (const string & name)
{
  if (_nameMap.find(name)==_nameMap.end())
    {
      _nameMap[name]=_lastTaxaId;
      _lastTaxaId++;
    }
  return(_nameMap[name]);
}


int bootstrap::idFromNameConst (const string & name) const
{
  NameMap_t::const_iterator i(_nameMap.find(name));
  if (i==_nameMap.end())
    {
      stringstream s;
      s<< "Can not find an Id for the taxa name \""<< name <<"\".  exiting";
      errorMsg::reportError(s.str());
    }
  //    else
  return(i->second);	// must use a diffrent setup for the const versions
}


void bootstrap::getTreeNodes(const tree& t)  const {
  vector<tree::nodeP> all;
  t.getAllNodes(all,t.iRoot());
  _id2TreeId.resize(all.size());
  _treeId2Id.resize(all.size());
  int l=_lastTaxaId;
  
  for (vector<tree::nodeP>::iterator i = all.begin(); i != all.end();++i)
        if (isLeaf(*i))
	  {
	    _id2TreeId[idFromNameConst((*i)->name())]=(*i)->id();
	    _treeId2Id[(*i)->id()]=idFromNameConst((*i)->name());
	    //	    cout << "[["<<idFromNameConst((*i)->name())<<" "<<(*i)->name()<<" "<<(*i)->id()<<"]]  ";
	  }
	else 
	  {
	    _id2TreeId[l]=(*i)->id();
	    _treeId2Id[(*i)->id()]=l;
	    //	    cout << "[.["<<l<<" "<<(*i)->name()<<" "<<(*i)->id()<<"].]  ";
	    ++l;
	  }  
  // cout<<endl;
}




int bootstrap::getIdFromTree(const int id) const
{
  //  if (id<_nTaxa)
  //      return(_treeId2Id[id]);
      return(_id2TreeId[id]);
  //else
      //  return id;
}

set<int> bootstrap::recursivlyFillBTVector(const tree::nodeP &n, vector<float> &v) const
{
  set<int> s;
  for(vector<tree::nodeP>::iterator i=n->sons.begin();i!=n->sons.end();++i) {
    set<int> sonSet(recursivlyFillBTVector(*i,v));
    s.insert(sonSet.begin(),sonSet.end());
  }
  if (n->sons.size()==0) {
    s.insert(idFromNameConst(n->name()));
  }
  set<int>::const_iterator sBeg(s.begin());
  set<int>::const_iterator sEnd(s.end());
  split sp(sBeg,sEnd,_nTaxa);
  v[n->id()]=((float)(_Splits.check(sp)))/_numTrees;
  return(s);
}


set<int> bootstrap::splitSubTreeRecursivly(const tree::nodeP &n, const bool isRoot=false)	// this function assumes that the tree is rooted not in a leaf
{
  set<int> s;
  for(vector<tree::nodeP>::iterator i=n->sons.begin();i!=n->sons.end();++i) {
    set<int> sonSet(splitSubTreeRecursivly(*i));
    s.insert(sonSet.begin(),sonSet.end());
  }
  if (n->sons.size()==0) {
    s.insert(idFromName(n->name()));
    //    cout << "   s "<< n->name()<<" "<< idFromName(n->name())<<endl;
  }
  else { // use this to keep out leafs
    if(!isRoot) {
      set<int>::const_iterator sBeg(s.begin());
      set<int>::const_iterator sEnd(s.end());
      split sp(sBeg,sEnd,_nTaxa);
      _Splits.add(sp);
    }
  }
  return(s);
}

void bootstrap::splitTree(const tree& T)// take tree, compute all splits and
// enter them into the Splits map
{
  if (!_nTaxa) _nTaxa=T.iLeaves();
  //    tree T(origTree);		// copy tree, as I will soon mutilate it.
  splitSubTreeRecursivly(T.iRoot(), true);
  _numTrees++;
}

void bootstrap::recursivlyPrintTreeNH(ostream &os, const tree::nodeP &nP, const vector<float> &v, const int nTaxa) const{
  if (nP->sons.size()==0) {
    os << nP->name();
    return;
  } else {
    os << "(";
    vector<tree::nodeP>::iterator i=nP->sons.begin();
    recursivlyPrintTreeNH(os,*i,v,nTaxa);
    for(++i;i!=nP->sons.end();++i) {
      os <<",";
      recursivlyPrintTreeNH(os,*i,v,nTaxa);
    }
    os << ")";
    if (v[nP->id()]>0)
      os << "["<<v[nP->id()]<<"]";
  }
}
