#ifndef ___BOOTSTRAP
#define ___BOOTSTRAP

#include <sstream>

#include "split.h"
#include "splitMap.h"
#include "tree.h"
#include "treeUtil.h"

using namespace libtree_v1_0_0;

class bootstrap {

public:
  
  typedef  vector<tree> treeVec;
  
  // constructor 
  explicit bootstrap (const treeVec& treevect);
  
  // optinal constructor
  explicit bootstrap (const string& filename);
  
  // give a tree and return fill it up with bootstrap values
  vector<float> GetWeightsToTree(const tree& inTree) const;

  
  // give a threshold >= 0.5 and get a concensus tree with all splits
  // that are more confident then the threshold.
  vector<float> bootstrap::thresholdTree(const tree& outTree, set<pair<int, int> > &edges, const float threshold=0.5) const;

  //here I need to define an itarator over splits, for 
  // GetSupportdSplits(float threshold)


  void print(ostream& sout = cout);
  void printTreeNH(ostream &os, const tree &t, const vector<float> &v) const;
  
  

private:

  void fillFromTreeVec(const treeVec& treevect);
  void computeReverse();
  int idFromName (const string & name) ;
  int idFromNameConst (const string & name) const;
  int getIdFromTree(const int id) const;

  void print_names(ostream &os) const;

  set<int> recursivlyFillBTVector(const tree::nodeP &n, vector<float> &v) const;
  set<int> splitSubTreeRecursivly(const tree::nodeP &n, const bool isRoot);	// this function assumes that the tree is rooted not in a leaf
  // take tree, compute all splits and enter them into the Splits map
  void splitTree(const tree& T);
  void recursivlyPrintTreeNH(ostream &os, const tree::nodeP &nP, const vector<float> &v, const int nTaxa) const;

  void getTreeNodes(const tree& t)  const ; // note that _allTree_nodes is mutable
  
  int _numTrees;			// total number of trees
  splitMap _Splits;
  typedef map<string,int> NameMap_t;
  NameMap_t _nameMap;
  int _lastTaxaId;
  int _nTaxa;
  splitMap::reverse_sMap_t _rmap;
  bool _reverseIsUpToDate;
  mutable  vector<int> _id2TreeId, _treeId2Id;
};



#endif  // ___BOOTSTRAP
