// DO: (1) ADD NUMBER OF DESCRETE CATEGORIES TO THE OPTIONS.
// DO: (2) ADD NUMBER OF RANDOM STARTS TO THE OPTIONS.
// DO: (3) CHENYSHEV PARAMETERS TO THE OPTIONS.
// DO: (4) ADD PARAMETER CONCERNING THE STARTING NJ TREE.
// DO: (5) ADD PARAMETER CONCERNING THE GAMMA PARAM OPTIMIZATION.

#include <iostream>
#include <fstream>
#include <cassert>
#include <string>
#include <stdio.h>
using namespace std;


#include "mainSemphy.h"
#include "jcDistance.h" // for NJ tree
#include "distanceTable.h" // for NJ tree
#include "nj.h" // for NJ tree
#include "brLenOptEM.h"// for bbl only
#include "semphySearchBestTree.h"
#include "logFile.h"
#include "readDatMatrix.h"
#include "datMatrixHolder.h"
#include "likelihoodComputation.h"

#include "mpiWrap.h"

#define TMPBUFSIZE 1024
void setLog(const char* logfilename, const int loglvl)
{
	if (!strcmp(logfilename,"-")||!strcmp(logfilename,""))
	{
		myLog::setLogOstream(&cout);
	}
	else
	{
	    ofstream* outLF = new ofstream;
	    outLF->open(logfilename);
	    if (!outLF->is_open()) {
	      	errorMsg::reportError("unable to open file for reading");
	    }
	    myLog::setLogOstream(outLF);
	}
	myLog::setLogLvl(loglvl);
	LOG(3,<<"START OF MUL LOG FILE\n\n");
}

mainSemphyNew::mainSemphyNew(int argc, char* argv[]) {

#ifdef _MPI
	if (_my_rank == 0){
#endif
		if (argc < 2) errorMsg::reportError("The program must get some parameters in the command line, use -h for list");
#ifdef _MPI
	}
#endif

	// STEP 1: WE PUT ALL THE COMMAND LINE INTO ARGC.

	// STEP 2: WE BUILD THE STRUCT OF SEMPHY INFO
	gengetopt_args_info args_info;
#ifdef _MPI
	int argCnt;
	char **argV;
	if (_my_rank==0) {
		argCnt = argc-4;
		argV = (char **) malloc(argCnt * sizeof(char **));
		for (int k=0;k<argCnt;k++) {
			argV[k] = (char*)malloc(100);
		}
        	for (int i=0; i<argCnt; i++) {
			strcpy(argV[i],argv[i]);
			// printf("argCnt=%d;argV[%d] = %s\n", argCnt, i, argV[i]);
		}
	}

	MPI_CALL(MPI_Bcast(&argCnt, 1, MPI_INT, 0, MPI_COMM_WORLD));
//	printf ("argCnt = %d, rank = %d\n", argCnt, _my_rank);
	
	if (_my_rank != 0)
	{
		argV = (char **) malloc(argCnt * sizeof(char **));
		for (int k=0;k<argCnt;k++) {
			argV[k] = (char*)malloc(100);
		}
	}
	
	for (int j=0; j<argCnt; j++) {
		MPI_CALL(MPI_Bcast(argV[j], 100, MPI_CHAR, 0, MPI_COMM_WORLD));
	}
#endif

#ifdef _MPI
	if (cmdline_parser (argCnt, argV, &args_info) != 0) {
#else
	if (cmdline_parser (argc, argv, &args_info) != 0) {
#endif
	  	errorMsg::reportError("error in command line parsing",1);
	}
#ifdef _MPI
	// printf ("Command parser ok!\n");	
	for (int k=0; k<argCnt; k++)
		free(argV[k]);
	free(argV);
#endif
	// AND WE CHECK TO SEE IF WE ARE ASKED FOR SOMETING WE CAN NOT DELIVER

	if(args_info.minimum_improvement_given)	errorMsg::reportError("minimum-improvement not yet implimented");
	if(args_info.seed_given)		errorMsg::reportError("random seen not yet implimented");
	if(args_info.maxDistance_given)		errorMsg::reportError("max distance not yet implimented");
	if(args_info.exact_given)		errorMsg::reportError("exact counts not yet implimented");

	// STEP 3: USING THE INFORMATION IN args_info WE CONSTRUCT OUR OBJECTS.

	// 3.1: GETTING THE ALPHABET RIGHT.
	alphabet*  alphP = NULL;
	switch (args_info.alphabet_arg) 
	  { // allwayes defined, with default
	  case 4:	
	    alphP = new nucleotide; 
	    break;
	  case 20: 
	    alphP = new amino; 
	    break;
	  default: errorMsg::reportError("error getting the alphabet");
	}

	// 3.2 GETTING THE SEQUENCE CONTAINER.
	sequenceContainer1G sc;
	ifstream ins;
	istream* inPtr = NULL;	

	string sequenceFileName(args_info.sequence_arg);
	if (sequenceFileName == "") sequenceFileName="-";
	if (sequenceFileName == "-") {
	  inPtr = &cin;
	} else{
	  ins.open(sequenceFileName.c_str());
	  if (! ins.is_open())
	    errorMsg::reportError("can not open sequace file");
	  inPtr = &ins;
	}
	istream& in = *inPtr;
	
	if      (strcmp(args_info.format_arg,"mase")==0)    {sc= maseFormat::read(in,alphP);}
	else if (strcmp(args_info.format_arg,"clustal")==0) {sc= clustalFormat::read(in,alphP);}
	else if (strcmp(args_info.format_arg,"molphy")==0)  {sc= molphyFormat::read(in,alphP);}
	else if (strcmp(args_info.format_arg,"phylip")==0)  {sc= phylipFormat::read(in,alphP);}
	else if (strcmp(args_info.format_arg,"fasta")==0)   {sc= fastaFormat::read(in,alphP);}
	else { errorMsg::reportError(" format not implemented yet in this version... ");}

	if (args_info.gaps_flag) {
		errorMsg::reportError("remove gap position is not implemented yet");
	} else {// gaps are changed to missing data...
		sc.changeGapsToMissingData();
	}
	
	// 3.3 GETTING THE TREE FROM THE TREE FILE - IF THERE IS SUCH.
	tree* myet = NULL;
	vector<char> * constraints = NULL;

	if (args_info.tree_given) // did we get a tree
	{
		string treeFileName(args_info.tree_arg);
	    vector<char> constraintsV;
	    myet = new tree(treeFileName,constraintsV);
	    constraints = new vector<char>(constraintsV);
	} 
	
	
	// 3.4 GETTING THE STOCHASTIC PROCESS.

	distribution *dist = NULL;
	if (args_info.gamma_given == false) 
	  dist =  new uniDistribution;
	else 
	  dist =  new gammaDistribution(args_info.gamma_arg,4);
		
	replacementModel *probMod=NULL;
	pijAccelerator *pijAcc=NULL;

	if (strcmp(args_info.model_arg,"day")==0) {
		probMod=new pupAll(datMatrixHolder::dayhoff);
		pijAcc = new chebyshevAccelerator(probMod);
	} else if (strcmp(args_info.model_arg,"jtt")==0) {
		probMod=new pupAll(datMatrixHolder::jones);
		pijAcc =new chebyshevAccelerator(probMod);
	} else if (strcmp(args_info.model_arg,"rev")==0) {
		probMod=new pupAll(datMatrixHolder::mtREV24);
		pijAcc = new chebyshevAccelerator(probMod);
	} else if (strcmp(args_info.model_arg,"wag")==0) {
		probMod=new pupAll(datMatrixHolder::wag);
		pijAcc = new chebyshevAccelerator(probMod);
	} else if (strcmp(args_info.model_arg,"cprev")==0) {
		probMod=new pupAll(datMatrixHolder::cpREV45);
		pijAcc = new chebyshevAccelerator(probMod);
	} else if (strcmp(args_info.model_arg,"nucjc")==0) {
		probMod=new nucJC;
		pijAcc = new trivialAccelerator(probMod);
	} else if (strcmp(args_info.model_arg,"aaJC")==0) {
		probMod=new aaJC;
		pijAcc = new trivialAccelerator(probMod);
	} else if ((strcmp(args_info.model_arg,"hky")==0) || (strcmp(args_info.model_arg,"k2p")==0)) {
		MDOUBLE ratio =args_info.ratio_arg;
		MDOUBLE Ap(0.25), Cp(0.25), Gp(0.25), Tp(0.25);
		sscanf(args_info.ACGprob_arg,"%lf,%lf,%lf", &Ap, &Cp, &Gp);
		Tp=1.0-(Ap+Cp+Gp);
		probMod=new hky(Ap,Cp,Gp,Tp,ratio);
		pijAcc = new trivialAccelerator(probMod);
	} else if ((args_info.alphabet_arg == 20) && 
		   (strncmp(args_info.model_arg,"file:",5)==0)) { // try to read the name as afile name
	  probMod=new pupAll(&(args_info.model_arg[5]));
	  pijAcc = new chebyshevAccelerator(probMod);
	} else {
       errorMsg::reportError("This replacement model is not yet available");
	}
	
	stochasticProcess sp(dist, pijAcc);
	if (probMod) delete probMod;
	if (pijAcc) delete pijAcc;
	if (dist) delete dist;

	// 3.5 GETTING THE REST OF THE DATA.
	string outputFileName = args_info.outputfile_arg;
	int numberOfRandomStart = 1;
	int maxBBLIT = args_info.max_bbl_iterations_arg;
	

	// 3.6 SETTING THE OUTPUT STREAM.

	//3.6.1 out
	ostream * outP=NULL;
	ofstream out_tmp;
	if (strcmp(args_info.outputfile_arg,"-")==0)  // cout
	  outP=&cout;
	else {
	  out_tmp.open(args_info.outputfile_arg);
	  outP=&out_tmp;
	}
	ostream&  out(*outP);


	// 3.6.2 logfile
	setLog(args_info.Logfile_arg, args_info.verbose_arg);


	// 3.7 IN CASE NO TREE IS GIVEN - WE START WITH THE NJ TREE.
	if (myet==NULL) {//NJ TREE
		jcDistance pd1(sc.alphabetSize()); // assumes all alphabets are the same size
		VVdouble disTab;
		vector<string> vNames;
		giveDistanceTable(&pd1,
						   sc,
						   disTab,
						   vNames);
		NJalg nj1;
		tree njTree = nj1.computeNJtree(disTab,vNames);
		myet = new tree (njTree);
	}

	// 3.8 "HAFSAKAT PIRSOMOT". // Hebrew Joke
#ifdef _OUTPUT
	printSemphyTitle(out);
#endif

	// here is the place to get more information from the instruction file //
#ifdef _MPI
	mpi_n = sc.seqLen() % _num_procs;   //if _num_proc smaller than this, the corresponding proc has one more task
	mpi_mean = (int)(sc.seqLen() / _num_procs);
	if (_my_rank < mpi_n) {
		++mpi_mean;
		mpi_spos = _my_rank * mpi_mean;
	}
	else
		mpi_spos = _my_rank * mpi_mean + mpi_n;
#endif
	if (!args_info.no_SEMPHY_flag) { // with SEMPHY
	  if (args_info.no_bbl_flag) { // with BBL
	    semphySearchBestTree(sc,*myet,sp,out,numberOfRandomStart);
	  } else {		// no BBL
	    errorMsg::reportError("using SEMPHY steps without branch length optimization makes no seance");
	  }
	} else { 		// no SEMPHY
	  if (args_info.no_bbl_flag) { // with BBL
	    MDOUBLE resL=brLenOptEM::optimizeBranchLength1G_EM(*myet,sc,sp,NULL,maxBBLIT,0.05);//maxIterations=1000,epsilon=0.05
	    out<<" the likelihood of the tree (only BBL) is: "<<resL<<endl;
	    out<<" the final tree: "<<endl;
	    myet->output(out);
	  } else {		// no BBL - likelihood computation only
	    MDOUBLE resL=likelihoodComputation:: getTreeLikelihoodAllPosAlphTheSame(*myet, sc, sp);
	    out<<" the likelihood of the tree (no bbl) is: "<<resL<<endl;
	  }
	}
	  

	delete alphP;
	delete myet;
	delete constraints;
}

