#ifndef ARRAY_HPP
#define ARRAY_HPP

#define _OPT
#ifdef _OPT
#include "pnlNodeValues.hpp"
#include "pnlStaticGraphicalModel.hpp"
#include "pnlTypeDefs.hpp"
#include "pnlNodeType.hpp"
#include "pnlModelDomain.hpp"

#include <stdexcept>
#include <iterator>
#include <algorithm>


PNL_BEGIN

template<class T>
class pnlArray 
{
public:
	T *elems;    // fixed-size array of elements of type T
	int count;

public:
	// type definitions
	typedef T              value_type;
	typedef T*             iterator;
	typedef const T*       const_iterator;
	typedef T&             reference;
	typedef const T&       const_reference;
	
	pnlArray (T* addr = 0, int ct = 0)	: elems(addr), count(ct) {}
	
	// iterator support
	iterator begin() { return elems; }
	const_iterator begin() const { return elems; }
	iterator end() { return elems+count; }
	const_iterator end() const { return elems+count; }
	
	// operator[]
	reference operator[](int i) { return elems[i]; }
	const_reference operator[](int i) const { return elems[i]; }
	
	// at() with range check
	reference at(int i) { return elems[i]; }
	const_reference at(int i) const { return elems[i]; }
	
	// front() and back()
	reference front() { return elems[0]; }
	const_reference front() const { return elems[0]; }
	reference back() { return elems[count-1]; }
	const_reference back() const { return elems[count-1]; }
	
	// size is constant
	int size() { return count; }
	
	// assign data to elems
	void assign(T *start, T *end) {memcpy (elems, start, sizeof(T) * count);}

	// direct access to data
	const T* data() const { return elems; }	

	// set address and count
	void setaddr(T* addr) {elems = addr; }
	void setcount (int n) {count = n; }

};

PNL_END
#endif
#endif
